/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_TRPCLONGSERVICE_TRPCLONGSERVICE_SKELETON_H
#define JAC_AP_TRPCLONGSERVICE_TRPCLONGSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/trpclongservice/trpclongservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace trpclongservice {
namespace skeleton {
namespace events
{
    using TRPCLong2CANEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::SWC2CAN_Signal_Values>;
    using Long2CANEvent = ara::com::internal::skeleton::event::EventAdapter<::SWC2CAN>;
    static constexpr ara::com::internal::EntityId TRPCLongServiceTRPCLong2CANEventId = 20663U; //TRPCLong2CANEvent_event_hash
    static constexpr ara::com::internal::EntityId TRPCLongServiceLong2CANEventId = 59938U; //Long2CANEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class TRPCLongServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultTRPCLong2CANEvent = TRPCLong2CANEvent.Initialize();
        ThrowError(resultTRPCLong2CANEvent);
        const ara::core::Result<void> resultLong2CANEvent = Long2CANEvent.Initialize();
        ThrowError(resultLong2CANEvent);
    }

    TRPCLongServiceSkeleton& operator=(const TRPCLongServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceId, mode)),
              TRPCLong2CANEvent(ptr->GetSkeleton(), events::TRPCLongServiceTRPCLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceId),
              Long2CANEvent(ptr->GetSkeleton(), events::TRPCLongServiceLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceSpec, mode)),
              TRPCLong2CANEvent(ptr->GetSkeleton(), events::TRPCLongServiceTRPCLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceSpec),
              Long2CANEvent(ptr->GetSkeleton(), events::TRPCLongServiceLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceContainer, mode)),
              TRPCLong2CANEvent(ptr->GetSkeleton(), events::TRPCLongServiceTRPCLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceContainer),
              Long2CANEvent(ptr->GetSkeleton(), events::TRPCLongServiceLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              TRPCLong2CANEvent(std::move(other.TRPCLong2CANEvent)),
              Long2CANEvent(std::move(other.Long2CANEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                TRPCLong2CANEvent = std::move(other.TRPCLong2CANEvent);
                Long2CANEvent = std::move(other.Long2CANEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::TRPCLong2CANEvent GetTRPCLong2CANEvent()
        {
            return std::move(TRPCLong2CANEvent);
        }
        events::Long2CANEvent GetLong2CANEvent()
        {
            return std::move(Long2CANEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = TRPCLong2CANEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = Long2CANEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::TRPCLong2CANEvent TRPCLong2CANEvent;
        events::Long2CANEvent Long2CANEvent;
    };
    explicit TRPCLongServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceId, mode)),
          TRPCLong2CANEvent(skeletonAdapter->GetSkeleton(), events::TRPCLongServiceTRPCLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceId),
          Long2CANEvent(skeletonAdapter->GetSkeleton(), events::TRPCLongServiceLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit TRPCLongServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceSpec, mode)),
          TRPCLong2CANEvent(skeletonAdapter->GetSkeleton(), events::TRPCLongServiceTRPCLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceSpec),
          Long2CANEvent(skeletonAdapter->GetSkeleton(), events::TRPCLongServiceLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit TRPCLongServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceContainer, mode)),
          TRPCLong2CANEvent(skeletonAdapter->GetSkeleton(), events::TRPCLongServiceTRPCLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceContainer),
          Long2CANEvent(skeletonAdapter->GetSkeleton(), events::TRPCLongServiceLong2CANEventId, ::jac::ap::trpclongservice::TRPCLongService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    TRPCLongServiceSkeleton(const TRPCLongServiceSkeleton&) = delete;

    TRPCLongServiceSkeleton(TRPCLongServiceSkeleton&&) = default;
    TRPCLongServiceSkeleton& operator=(TRPCLongServiceSkeleton&&) = default;
    TRPCLongServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          TRPCLong2CANEvent(token.GetTRPCLong2CANEvent()),
          Long2CANEvent(token.GetLong2CANEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~TRPCLongServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&TRPCLongServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::TRPCLong2CANEvent TRPCLong2CANEvent;
    events::Long2CANEvent Long2CANEvent;
};
} // namespace skeleton
} // namespace trpclongservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_TRPCLONGSERVICE_TRPCLONGSERVICE_SKELETON_H
